package ch.epfl.tpp;

import java.io.IOException;
import java.io.OutputStream;

public class Base64OutputStream extends OutputStream {
    private final static String encodingString = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    
    final OutputStream encodedStream;
    
    /* Invariant: buffer contains, in its least-significant bits, bufferLen
     * decoded bits that still have to be encoded and sent to the encoded
     * stream. Since bits are received in chunks of 8 bits and sent to the
     * client in chunks of 6, bufferLen will rotate between the values 0, 4, 2.
     */
    int buffer = 0;
    int bufferLen = 0;
    int inputBytesCount = 0;
    
    public Base64OutputStream(OutputStream underlyingStream) {
        this.encodedStream = underlyingStream;
    }
    
    @Override
    public void write(int b) throws IOException {
        assert 0 <= b && b <= 0xFF;
        assert bufferLen == 0 || bufferLen == 2 || bufferLen == 4;
        buffer = (buffer << 8) | b;
        bufferLen += 8;
        while (bufferLen >= 6) {
            // We have 6 encoded bits available, encode them and write the
            // resulting 8 bits (as a character) to the encoded stream.
            bufferLen -= 6;
            encodedStream.write(encodingString.charAt((buffer >> bufferLen) & 0b111111));
        }
        ++inputBytesCount;
    }
    
    @Override
    public void close() throws IOException {
        // Finish by adding enough 0 bits to have 6 bits to encode.
        if (bufferLen > 0)
            encodedStream.write(encodingString.charAt((buffer << (6 - bufferLen)) & 0b111111));
        // Add padding to ensure the output length is a multiple of 4.
        switch (inputBytesCount % 3) {
        case 1: encodedStream.write('='); // no break!
        case 2: encodedStream.write('='); // no break!
        case 0:
        }
        encodedStream.close();
    }
}
