package ch.epfl.tpp;

import static org.junit.Assert.*;

import java.io.*;

import org.junit.Test;

public class Base64Test {
    private String encodeString(String string) throws IOException {
        ByteArrayOutputStream baStream = new ByteArrayOutputStream();
        Base64OutputStream b64Stream = new Base64OutputStream(baStream);
        for (int i = 0; i < string.length(); ++i) {
            char c = string.charAt(i);
            b64Stream.write(c);
        }
        b64Stream.close();
        return baStream.toString("US-ASCII");
    }

    @Test
    public void testKnownStringsEncoding() throws IOException {
        assertEquals("Yg==", encodeString("b"));
        assertEquals("YmE=", encodeString("ba"));
        assertEquals("YmFz", encodeString("bas"));
        assertEquals("YmFzZQ==", encodeString("base"));
        assertEquals("YmFzZTY=", encodeString("base6"));
        assertEquals("YmFzZTY0", encodeString("base64"));
        assertEquals("YmFzZTY0IQ==", encodeString("base64!"));
        assertEquals("TnVsIG4nZXN0IHBsdXMgZXNjbGF2ZSBxdWUgY2VsdWkgcXVpIHNlIGNyb2l0IGxpYnJlIHNhbnMgbCdldHJl",
                encodeString("Nul n'est plus esclave que celui qui se croit libre sans l'etre"));
    }

    @Test
    public void testAllBytesEncoding() throws IOException {
        String enc = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
        int[] dec = { 0x00, 0x10, 0x83, 0x10, 0x51, 0x87, 0x20, 0x92, 0x8b,
                0x30, 0xd3, 0x8f, 0x41, 0x14, 0x93, 0x51, 0x55, 0x97, 0x61,
                0x96, 0x9b, 0x71, 0xd7, 0x9f, 0x82, 0x18, 0xa3, 0x92, 0x59,
                0xa7, 0xa2, 0x9a, 0xab, 0xb2, 0xdb, 0xaf, 0xc3, 0x1c, 0xb3,
                0xd3, 0x5d, 0xb7, 0xe3, 0x9e, 0xbb, 0xf3, 0xdf, 0xbf };

        ByteArrayOutputStream baStream = new ByteArrayOutputStream();
        Base64OutputStream b64Stream = new Base64OutputStream(baStream);
        for (int b: dec)
            b64Stream.write(b);
        b64Stream.close();
        assertEquals(enc, baStream.toString("US-ASCII"));
    }

    private String decodeString(String string) throws IOException {
        ByteArrayInputStream baStream = new ByteArrayInputStream(string.getBytes("US-ASCII"));
        Base64InputStream b64Stream = new Base64InputStream(baStream);
        StringBuilder b = new StringBuilder();
        int c;
        do {
            c = b64Stream.read();
            if (c != -1)
                b.append((char)c);
        } while (c != -1);
        b64Stream.close();
        return b.toString();
    }

    @Test
    public void testKnownStringsDecoding() throws IOException {
        assertEquals("b", decodeString("Yg=="));
        assertEquals("ba", decodeString("YmE="));
        assertEquals("bas", decodeString("YmFz"));
        assertEquals("base", decodeString("YmFzZQ=="));
        assertEquals("base6", decodeString("YmFzZTY="));
        assertEquals("base64", decodeString("YmFzZTY0"));
        assertEquals("base64!", decodeString("YmFzZTY0IQ=="));
        assertEquals("Nul n'est plus esclave que celui qui se croit libre sans l'etre",
                decodeString("TnVsIG4nZXN0IHBsdXMgZXNjbGF2ZSBxdWUgY2VsdWkgcXVpIHNlIGNyb2l0IGxpYnJlIHNhbnMgbCdldHJl"));
    }

    @Test
    public void testAllBytesDecoding() throws IOException {
        String enc = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
        int[] dec = { 0x00, 0x10, 0x83, 0x10, 0x51, 0x87, 0x20, 0x92, 0x8b,
                0x30, 0xd3, 0x8f, 0x41, 0x14, 0x93, 0x51, 0x55, 0x97, 0x61,
                0x96, 0x9b, 0x71, 0xd7, 0x9f, 0x82, 0x18, 0xa3, 0x92, 0x59,
                0xa7, 0xa2, 0x9a, 0xab, 0xb2, 0xdb, 0xaf, 0xc3, 0x1c, 0xb3,
                0xd3, 0x5d, 0xb7, 0xe3, 0x9e, 0xbb, 0xf3, 0xdf, 0xbf };

        ByteArrayInputStream baStream = new ByteArrayInputStream(enc.getBytes("US-ASCII"));
        Base64InputStream b64Stream = new Base64InputStream(baStream);
        for (int b: dec)
            assertEquals(b, b64Stream.read());
        b64Stream.close();
    }

    @Test(expected=java.io.IOException.class)
    public void testInvalidEncodedChar() throws IOException {
        ByteArrayInputStream baStream = new ByteArrayInputStream("@".getBytes("US-ASCII"));
        Base64InputStream b64Stream = new Base64InputStream(baStream);
        b64Stream.read();
        b64Stream.close();
    }

    @Test(expected=java.io.IOException.class)
    public void testInvalidRemainingBits() throws IOException {
        ByteArrayInputStream baStream = new ByteArrayInputStream("++==".getBytes("US-ASCII"));
        Base64InputStream b64Stream = new Base64InputStream(baStream);
        b64Stream.read();
        b64Stream.read();
        b64Stream.close();
    }
}
